BEGIN {
    $ENV{ATRIA_FORCE_GUI} = 1 unless exists $ENV{ATRIA_FORCE_GUI};
    my $name = $ENV{LOGNAME} || $ENV{USERNAME};
    require ClearCase::ClearPrompt;
    ClearCase::ClearPrompt->import('+STDERR', "+STDERR=$name",
				'+WARN', "+WARN=$name",
				'+DIE', "+DIE=$name");
}

# All of the next three msgs should be "dialoged" AND emailed.
warn qq(This is a warning\n);		# explicit warning from this process
system q(perl nosuchscript);		# error msg generated by child process
die qq(This is a fatal error);		# explict error msg from this process

print "didn't die!?\n";			# we should never get here
exit 11;				# unique error code in case we did
